@@ -546,6 +546,9 @@ enum Syntax_error	*err_level;    /* return: syntax errors encountered in page */
 			}
 		     }
 		  } else {    /* numerical */
+		     buf_add_char('\0');    /* make sure string is terminated */
+		     --text_buf_len;    /* ...but don't keep the '\0' when appending more text! */
+
 		     /* extract number from numerical character reference */
 		     if(text_buf[amp_pos+2]=='x')    /* hex number */
 			replace=(int)strtol(&text_buf[amp_pos+3], NULL, 16);
@@ -564,7 +567,8 @@ enum Syntax_error	*err_level;    /* return: syntax errors encountered in page */
 			fprintf(stderr, "memory allocation error while syntax parsing (in function parse_syntax)\n");
 			exit(1);
 		     }
-		  }
+		  } else
+		     replace=0;    /* drop invalid replace */
 
 		  parse_mode=prev_mode_amp;    /* now back to previous mode... */
 		  if(!(in==';' && replace))    /* ';' ending a valid reference is swallowed; in all other cases, handle ending char */
@@ -732,7 +736,7 @@ enum Syntax_error	*err_level;    /* return: syntax errors encountered in page */
 		     exit(1);
 		  }
 		  insert_buf(&last_el->attr[last_el->attr_count-1].name.str);    /* set attribute name to the name just extracted */
-		  last_el->attr[last_el->attr_count-1].value.str="";    /* make sure we have a value, even if no real value can be stored for some reason */
+		  last_el->attr[last_el->attr_count-1].value.str=NULL;    /* make sure we have a definend (non-)value, if no real value will be stored for some reason */
 
 		  parse_mode=PM_ATTR_NAME_END;
 		  recycle=1;    /* may need additional processing */
@@ -1393,7 +1397,12 @@ enum Syntax_error	*err_level;    /* return: syntax errors encountered in page */
    else if (cur_el->parent!=NULL)    /* inside some element (only global element has no parent) */
       if(!input->user_break)
 	 html_error(input, 0, 0, "Unexpected end of file. (Unclosed elements.)", NULL);
-#endif    /* XHTML_ONLY */
+#else    /* not XHTML_ONLY */
+   else if(text_buf_len!=0) {    /* have some pending content */
+      last_el=add_element();    /* -> create dummy tag */
+      insert_buf(&last_el->content);    /* and save to it */
+   }
+#endif    /* not XHTML_ONLY */
 
    last_el->list_next=tree_top;
 
